function MenuItem(title, value, selected, icon) {
	this.title = title ? title : typeof(value.toString) === "function" ? value.toString() : value + "";
	this.value = value;
	this.icon = icon;
	this.selected = selected && selected === "true" ? true : false;
	
	this.select = function () {	this.selected = true; }
	this.deselect = function () { this.selected = false; }
	this.toggle = function() { this.selected = !this.selected; }
}

function SubMenu(id, title, multiselectable, icon) {
	this.id = id;
	this.title = title;
	this.multiselectable = multiselectable ? true : false;
	this.icon = icon;
	this.items = [];
	
	this.item = function (index) { return this.items[index]; }
	
	// Palauttaa tosi, jos annetussa indeksissä oleva itemi on valittuna.
	this.isSelected = function (index) {
		var item = this.items[index];
		return item && item.selected ? true : false;
	}
		
	this.deselectAll = function () {
		for (var i = 0; i < this.items.length; ++i) this.items[i].deselect();
	}
	
	this.selectAll = function () {
		for (var i = 0; i < this.items.length; ++i) this.items[i].select();
	}
	
	// Palauttaa valitun(-tujen) menuitemeiden value-muuttujat
	this.selected = function () {
		var values = [];
		for (var i = 0; i < this.items.length; ++i) if (this.items[i].selected) values.push(this.items[i].value);
		if (this.multiselectable) return values;
		else return values.length === 0 ? null : values[0];
	}
	
	this.selectedIndex = function () {
		for (var i = 0; i < this.items.length; ++i) if (this.items[i].selected) return i;
		return -1;
	}
	
	this.selectedIndexes = function () {
		var arr = [];
		for (var i = 0; i < this.items.length; ++i) if (this.items[i].selected) arr.push(i);
		return arr;
	}
	
	this.select = function (menuItemValue) {
		if (!this.multiselectable) this.deselectAll();

		for (var i = 0; i < this.items.length; ++i) {
			if (this.items[i].value === menuItemValue) {
				this.items[i].select();
				break;
			}
		}
	}
	
	this.setSelected = function (selectedIndexes) {
		this.deselectAll();
		if (!(selectedIndexes instanceof Array)) {
			this.items[selectedIndexes].select();
			return;
		}
		
		for (var i = 0; i < selectedIndexes.length; ++i) {
			if (this.items.length <= selectedIndexes[i]) {
				this.deselectAll(); // Yritetään asettaa valituksi viallinen indeksi => nollataan kaikki valinnat ja palataan
				return;
			}
			this.items[selectedIndexes[i]].select();
		}
	}
		
	this.add = function (item) {
		this.items.push(item);
	}
}

(function($) {

	$.widget("ksenos.kmenu", {
		/*
		Eventit: 
			'changed' - Kutsutaan, kun jotain valikon arvoa on muutettu
			'selected' - Kutsutaan, kun päämenun itemi on valittu
		*/
		options: {
			id: "menu",
			title: "title",
			submenus: [],
			items: []
		},

		// Palauttaa alivalikon id:n perusteella
		submenu: function (id) {
			for (var i = 0; i < this.options.submenus.length; ++i) if (this.options.submenus[i].id === id) return this.options.submenus[i];
			return null;
		},

		// Palauttaa menun itemin indeksin perusteella
		item: function (index) { return this.options.items[index]; },
		
		add: function (submenu) { this.options.submenus.push(submenu); },

		addItem: function (item) { this.options.items.push(item); },

		// Palauttaa tosi, jos menu on näkyvissä
		isVisible: function () { return this.element.children().is(':visible'); },
		
		// Piilottaa menun välittömästi
		hide: function () { this.element.children().menu().slideUp(0); },
		
		// Piilottaa menun animaatiolla
		slideUp: function () { this.element.children().menu().hide(); },
		
		// Tuo menun näkyviin
		slideDown: function () { this.element.children().menu().show(); },

		showHide: function () { this.isVisible() ? this.slideUp() : this.slideDown() },

		// Yrittää ladata local storageen tallennetun menun tilan
		load: function () {
			if (!localStorage || JSON === undefined) return;
			
			for (var i = 0; i < this.options.submenus.length; ++i) {
				var submenu = this.options.submenus[i];
				var value = localStorage[submenu.id];
				var selected = undefined;
				try {
					selected = JSON.parse(value);
				} catch (e) {
					// Asetusta ei ollut talletettuna localStorageen. Asetetaan selectediin tällä hetkellä valitut asetukset,
					// että niistä lähetettäisiin changed-signaalit.
					selected = submenu.selectedIndexes();
				}

				if (selected instanceof Array) {
					submenu.deselectAll();
					for (var j = 0; j < selected.length; ++j) {
						this._onSelected(selected[j], submenu.id);
					}
				}
				
				this.update(submenu);
			}
		},
		
		// Tallentaa menun tilan local storageen
		_save: function () {
			if (!localStorage || JSON === undefined) return;	
			for (var i = 0; i < this.options.submenus.length; ++i) {
				localStorage[this.options.submenus[i].id] = JSON.stringify(this.options.submenus[i].selectedIndexes());
			}
		},
		
		_onSelected: function (index, submenuId) {
			// Tarkastetaan valittiinko päämenun itemi
			if (submenuId === this.options.id) {
				this._trigger("selected", null, this.item(index));
				return;
			}

			var submenu = this.submenu(submenuId);
			if (!submenu) return;

			var item = submenu.item(index);
			if (!item) return;

			if (!submenu.multiselectable) submenu.deselectAll();
			item.toggle();

			this.update(submenu);
			this._trigger("changed", null, submenu);
		},

		_create: function () {
			function createListElement(submenuid, index, title, isSelected, icon) {
				var li = $("<li></li>").addClass("menu-item").attr("data-submenuid", submenuid).attr("data-index", index);				
				if (!icon) icon = "check";
				var i = $("<span></span>").addClass("ui-icon").addClass("ui-icon-" + icon);
				if (!isSelected) i.hide();
				li.append(i).append(title);
				return li;
			};
		
			var ul = $("<ul></ul>");
			for (var j = 0; j < this.options.submenus.length; ++j) {
				var submenu = this.options.submenus[j];
				if (submenu === undefined) continue;
			
				var m = $("<li></li>").addClass("submenu").attr("data-id", submenu.id);
				if (submenu.icon) m.append($("<span></span>").addClass("ui-icon").addClass("ui-icon-" + submenu.icon))
				m.append(submenu.title);
				
				if (submenu.items.length > 0) {
					var submenuUl = $("<ul></ul>");
					for (var i = 0; i < submenu.items.length; ++i) {
						var item = submenu.items[i];
						submenuUl.append(createListElement(submenu.id, i, item.title, item.selected, item.icon));
					}
					m.append(submenuUl);
				}
				ul.append(m);				
			}

			for (var i = 0; i < this.options.items.length; ++i) {
				var item = this.options.items[i];
				ul.append(createListElement(this.options.id, i, item.title, item.selected, item.icon));
			}

			var that = this;
			var m = this.element.append(ul);
			m.children().menu({
				select: function (event, ui) {
					var index = ui.item.attr("data-index");
					var submenuId = ui.item.attr("data-submenuid");
					that._onSelected(index, submenuId);
					that._save();
				}
			});
			this.hide();
		},
		
		// Päivittää alimenun valittu/ei-valittu ikonit
		update: function (submenu) {
			$(".submenu[data-id=" + submenu.id + "]").find("ul").children("li").each(function () {
				var item = submenu.item($(this).attr("data-index"));
				if (item.selected) $(this).children("span").show();
				else $(this).children("span").hide();
			});
		}
	});

})(jQuery);
