(function($) {

	$.widget("ksenos.timeline", {
	
		options: {
			isEnabled: true,
			isError: false,
			
			disabledText: "-"
		},
		
		_create: function () {
			var that = this;
			$(window).resize(function() { that._createTimeline(); });
		
			this.isSeeking = false;
			
			this.height = 40; // Timelinen koko päätellään css:n heightin perusteella ja se asetetaan tähän. Eli tämän muuttaminen ei muuta korkeutta.
			this.visibleWidth = 0; // Aikajanan näkyvän alueen leveys
			this._width = 0; // Aikajanan todellinen leveys
			this._offset = 0;
			
			this.backgroundColor = "rgb(43, 47, 56)";
			this.timeTextColor = "rgba(255, 255, 255, 0.8)";
			this.barColor = "rgba(0, 0, 0, 0.5)";
			this.gradientStartColor = "rgba(0, 0, 0, 1)";
			this.gradientEndColor = "rgba(0, 0, 0, 0)";
			this.noTimeColor = "rgba(255, 255, 255, 0.1)";
			this.recAreaColor = "rgba(66, 156, 216, 1.0)";
			
			this.markColor = "rgb(255, 255, 255)";
			this.markWidth = 1;
			
			this.canvas = null;
			this.ctx = null;
			this.dummyCanvas = null;
			this.dummyCtx = null;
			this.overlayCtx = null;
			this.stateCtx = null;
			this.timeCtx = null;
			
			this._rangeBegin = -1;
			this._rangeEnd = -1;
			
			this._bounds = null; // Taulukko aikaväleistä, joilla on tallennetta
			this._begin = null; // Timelinen alku ajankohta
			this._end = null; // Timelinen loppu ajankohta
			this._boundsAfterScale = null;
			this._beginAfterScale = null;
			this._endAfterScale = null;
			
			this._time = null;
			this._state = ""; // toiston tilaa kuvaava teksti "Live", "Stopped", jne.
			this._live = false; // Ollaanko live-tilassa?
			this._playSpeed = 0;
			this._dragSpeed = 0;
			this._timeTextColor = this.timeTextColor;
			
			this._createTimeline();
		},
		
		_createTimeline: function () {
			var that = this;
			
			// Muutetaan kohde-elementti näkyvän alueen kokoiseksi
			var target = this.element;
			target.html("");
			
			this._offset = 0;
			this.visibleWidth = parseInt(target.css("width"));
			this._width = this.visibleWidth * 2; // Ei voi muuttaa tai hajottaa kaiken
			this.height = parseInt(target.css("height"));
			target.css("background-color", "#000000");

			var zIndex = 0;
			var vw = this.visibleWidth;
			function addCanvas(left, width, height) {
				zIndex++;
				var c = $("<canvas class='timeline-canvas' style='position: absolute; "
					+ "left: " + left + "px; overflow: hidden; "
					+ "z-index: " + zIndex 
					+ ";' width='" + width + "' height='" + height + "'></canvas>");
				target.append(c);
				return c.get(0);
			}

			// Lisätään canvas itse aikajanalle
			this.canvas = addCanvas((-this._width / 4), this._width, this.height);
			this.ctx = this.canvas.getContext('2d');
			this.dummyCanvas = document.createElement('canvas');
			this.dummyCanvas.width = this.canvas.width;
			this.dummyCanvas.height = this.canvas.height;
			this.dummyCtx = this.dummyCanvas.getContext('2d');
			
			// Lisätään canvas, johon piirretään gradientit ja merkki toistokohdan näyttämiselle
			this.overlayCtx = addCanvas(0, this.visibleWidth, this.height).getContext('2d');
			this.drawOverlay();
			
			// Lisätään vasempaan reunaan canvas, johon piirretään toiston tila
			this.stateCtx = addCanvas(0, 100, this.height).getContext('2d');
			this.stateCtx.font= "14px Arial";
			this.stateCtx.fillStyle = this._timeTextColor;
			
			// Lisätään elementti raahauksen käsittelyä varten
			var drag = $(addCanvas(0, this.visibleWidth, this.height));

			// Lisätään oikeaan reunaan canvas, johon piirretään aika
			var timeCanvas = addCanvas(this.visibleWidth - 80, 80, this.height); 
			this.timeCtx = timeCanvas.getContext('2d');
			this.timeCtx.font= "14px Arial";
			this.timeCtx.fillStyle = this._timeTextColor;
			$(timeCanvas).click(function () { that._trigger("dateset"); });	

			var paused = false;
			var lastX = 0;
			var startX = 0;
			drag.draggable({
				axis: "x",
				start: function (event, ui) {
					lastX = 0;
					startX = 0;
					that.isSeeking = true;
					that._trigger("seekingstarted");
				},
				stop: function (event, ui) {
					drag.css("left", "0px"); // Nollataan raahaimen sijanti
					that._updateTime();
					that.isSeeking = false;
					that._trigger("seekingfinished");
				},
				drag: function (event, ui) {
					// Jos ollaan raahattu yli puolet näkyvästä alueesta, päivitetään aikajana
					if (Math.abs(ui.position.left - startX) > that.visibleWidth / 2) {
						startX = ui.position.left;
						that._trigger("seekingfinished");
						that._trigger("seekingstarted");
						return;
					}
					
					that.move(lastX - ui.position.left);
					that._dragSpeed = lastX - ui.position.left;
					lastX = ui.position.left;
					
					if (!paused) {
						that._updateTime();
						paused = setTimeout(function () { paused = false; that._dragSpeed = 0; }, 100);
					}
				}
			});
			
			var scrolling = false;
			var totalDelta = 0;
			drag.bind("mousewheel", function (event) {
				totalDelta += event.deltaY;
				if (!scrolling) {
					scrolling = setTimeout(
						function () {
							scrolling = false;
							that._trigger("zoomed", null, totalDelta);
							totalDelta = 0;
						}, 150); // Kerätään skrollauksia 150ms ja zoomataan sitten
				}
			});
			
			var moveFps = 60;
			var moveDuration = 400;
			drag.on("click", function (event) {
				// Siirrytään enintään tämänhetkiseen aikaan asti
				var moveTarget = event.clientX < (that.currentPosition() - that.visibleWidth / 2) ? event.clientX : (that.currentPosition() - (that._width / 4) - that._offset);
				moveTarget -= that.visibleWidth / 2;
				var moveInterval = moveTarget / (moveDuration / 1000.0 * moveFps);
				
				that._trigger("seekingstarted");
				
				var updateTimer = 0;
				function moveAnim() {
					that.move(moveInterval);
					updateTimer += 1000.0 / moveFps;
					if (updateTimer > 200) { // Päivitetään siirtyminen 200 millisekunnin välein
						that._updateTime();
						updateTimer = 0;
					}
					moveTarget -= moveInterval;
					if ((moveInterval < 0 && moveTarget >= 0) || (moveInterval > 0 && moveTarget <= 0)) {
						that._updateTime();
						that._trigger("seekingfinished");
					} else {
						setTimeout(moveAnim, 1000.0 / moveFps);
					}
				}
				setTimeout(moveAnim, 1000.0 / moveFps);
			});

			this._trigger("created");
		},
		
		setEnable: function (state) {
			this.options.isEnabled = state;
			this.draw();
			this.drawTime();
			this.drawState();
			this.drawOverlay();
		},

		setError: function (state) {
			var wasError = this.options.isError;
			this.options.isError = state;
			if (!wasError && this.options.isError) this.animateError();
		},

		setDisabledText: function (text) {
			this.options.disabledText = text;
			this.setEnable(this.options.isEnabled);
		},

		set: function (bounds, begin, end) {
			if (this.isScaling()) { // Jos ollaan skaalaamassa, tallennetaan rajat ja muutetaan ne vasta, kun skaalaus on valmis
				this._boundsAfterScale = bounds;
				this._beginAfterScale = begin;
				this._endAfterScale = end;
				return;
			}
			
			this.resetScale();
			
			this._bounds = bounds;
			this._begin = begin;
			this._end = end;
			
			this._resetOffset();
		},
			
		// Lisää uusia boundsseja. End on uusi timelinen loppuaika.
		add: function (bounds, end) {
			if (this.isScaling() || !this._bounds) return;
			
			var duration = end - this._end; // Lasketaan kuinka monta sekuntia aikajanaan lisättiin
			if (duration < 0) return;

			var begin = this._begin + duration; // Aikajanan uusi alkuaika
			
			// Tarkastetaan alkaako seuraava lisättävä boundssi heti tällä hetkellä viimeisen jälkeen
			if (this._bounds[this._bounds.length - 1] === bounds[0]) {
				// Alkoi heti viimeisen jälkeen eli yhdistetään
				this._bounds[this._bounds.length - 1] = bounds[1]; // Asetetaan vanhaan boundsiin uusi loppuaika...
				bounds.splice(0, 2); // ...ja poistetaan uusi boundssi.
			}
			
			var bounds = this._bounds.concat(bounds); // Uusi taulukko, jossa on uudet boundssit vanhojen perässä
			for (var i = 0; i < bounds.length; i += 2) { // Poistetaan boundssit, jotka loppuu ennen aikajanan alkua
				if (bounds[i + 1] < begin) {
					bounds.splice(0, 2);
					i -= 2;
				} else break;
			}
			
			this.set(bounds, begin, end);
		},

		// Asettaa aikajanan keskipisteen annettun ajan kohdalle
		setTime: function (time) {		
			if (!this._bounds || this.isScaling()) return;

			var pixels = this.toPixels(time - this._begin) - this.width() / 2 - this._offset;
			this.move(pixels);
			
			this.drawTime();
			this.draw();

			// Kutsutaan onApproachingEdgeä, kun ollaan aikajanan ensimmäisessä tai viimeisessä viidenneksessä
			var oneFifth = (this._end - this._begin) / 5;
			if (time - this._begin < oneFifth || this._end - time < oneFifth) this._trigger("approachingedge");
		},

		setState: function (state) {
			this._state = state;
			this.drawState();
		},
		
		isLive: function () { return this._live; },
		setLive: function (isLive) {
			this._live = isLive;
		},
		
		playSpeed: function () { return this._playSpeed; },
		setPlaySpeed: function (speed) {
			this._playSpeed = speed;
		},
		
		rangeBegin: function () { return this._rangeBegin; },
		setRangeBegin: function () {
			if (this._rangeEnd !== -1 && this._rangeEnd < this.time()) this._rangeEnd = this.time();
			else this._rangeBegin = this.time();
			this.draw();
		},
		rangeEnd: function () { return this._rangeEnd; },
		setRangeEnd: function () {
			if (this._rangeBegin !== -1 && this._rangeBegin > this.time()) this._rangeBegin = this.time();
			else this._rangeEnd = this.time();
			this.draw();
		},
		clearRange: function () {
			this._rangeBegin = -1;
			this._rangeEnd = -1;
			this.draw();
		},

		scale: 1,
		_targetScale: -1,
		_scaleDuration: 400, // millisekunteissa
		_scaleTimeLeft: 0,
		resetScale: function () {
			if (this._targetScale === -1) return;
			this._targetScale = -1;
			this._setScale(1, true);
		},
		isScaling: function () { return Math.abs(this._targetScale + 1) > 0.01; },
		setScale: function (scale) {
			if (this.isScaling()) {
				this._targetScale *= scale;
				this._scaleTimeLeft += 100;
				return;
			}
			
			if (Math.abs(this.scale - scale) < 0.1) return; // Ollaan jo halutulla scale-tasolla

			this._targetScale = scale;
			this._scaleTimeLeft = this._scaleDuration;
			var start = null;
			var that = this;

			function scaleAnim(timestamp) {
				// Jos targetScale on -1, on skaalaus lopetettu
				if (Math.abs(that._targetScale + 1) < 0.01) return;
				
				if (start === null) start = timestamp;
				var progress = (timestamp - start);
				if (progress === 0) {
					requestAnimFrame(scaleAnim);
					return;
				}
				start = timestamp;
				var scaleInterval = (that.scale - that._targetScale) / (that._scaleTimeLeft / progress);
				that._scaleTimeLeft -= progress;
				
				var nextScale = that.scale - scaleInterval;
				if (that._scaleTimeLeft <= 0 ||
					(scaleInterval > 0 && nextScale < that._targetScale) ||
					(scaleInterval < 0 && nextScale > that._targetScale)) {
					// Asetettava scale ei ollut enää rajojen sisällä eli ollaan saavutettu target scale
					
					// Tarkastetaan onko rajoja koitettu muuttaa skaalaamisen aikana
					if (that._boundsAfterScale) {
						that._targetScale = -1;
						that._setScale(1, true);
						that.set(that._boundsAfterScale, that._beginAfterScale, that._endAfterScale);
						that._boundsAfterScale = that._beginAfterScale = that._endAfterScale = null;
					} else {
						that._setScale(that._targetScale);
					}
				} else {
					that._setScale(nextScale);
					requestAnimFrame(scaleAnim);
				}
			};
			requestAnimFrame(scaleAnim);
		},
		_setScale: function (scale, dontDraw) {
			if (scale <= 0) return;
				
			var that = this;
			
			var oldPixelPosition = this.pixelPosition(this.time());
			this.scale = scale;
			var newPixelPosition = this.pixelPosition(this.time());
			var w = newPixelPosition - oldPixelPosition;
			
			this.ctx.translate(-w, 0);
			this._offset += w;
			if (!dontDraw) this.draw();
		},
		
		_updateTime: function () {
			this.drawTime();
			this._trigger("timechanged", null, this._time.getTime());
		},
		
		width: function () { return this._width * this.scale; },
		
		// Palauttaa tämän hetkisen ajan
		time: function () {	return this._time.getTime() / 1000; },

		_resetOffset: function () {
			var realTime = (!this._time) ? (this._begin + this.length() / 2) : this.time();
			
			this.ctx.translate(this._offset, 0);
			this._offset = 0;
			this._time = new Date((this._begin + this.length() / 2) * 1000);
			
			this.setTime(realTime);
		},
			
		move: function (pixels) {
			this._time = new Date(this._time.getTime() + this.toSeconds(pixels) * 1000);
			this._offset += pixels;

			this.dummyCtx.clearRect(0, 0, this._width, this.height);
			this.dummyCtx.drawImage(this.canvas, 0, 0);
			
			this.ctx.save();
			this.ctx.setTransform(1, 0, 0, 1, 0, 0);
			this.ctx.clearRect(0, 0, this._width, this.height);
			this.ctx.translate(-pixels, 0);
			this.ctx.drawImage(this.dummyCanvas, 0, 0);
			this.ctx.restore();
			
			this.ctx.translate(-pixels, 0);
		},
		
		// Palauttaa annetun ajan sijainnin x-akselilla
		pixelPosition: function (time) {
			return this.toPixels(time - this._begin);
		},
		
		toTime: function (pixelPosition) {
			return this._begin + this.toSeconds(pixelPosition);
		},
		
		// Palauttaa kuinka monta pikseliä annettu sekuntimäärä vastaa
		toPixels: function (seconds) {
			return (this.width() / this.length()) * seconds;
		},
		
		// Palauttaa kuinka montaa sekuntia annettu pikselimäärä vastaa
		toSeconds: function (pixels) {
			return (this.length() / this.width()) * pixels;
		},
		
		// Palauttaa tosi, jos annettu aika on aikajanan rajojen sisällä
		insideBounds: function (time) {
			return time > this._begin && time < this._end;
		},
		
		// Palauttaa aikajanan keston sekunneissa
		length: function () {
			return this._end - this._begin;
		},
		
		// Palauttaa tämänhetkisen ajan sijainnin
		currentPosition: function  () {
			return this.pixelPosition(new Date().getTime() / 1000);
		},

		draw: function () {
			this.ctx.save();
			this.ctx.setTransform(1, 0, 0, 1, 0, 0);
			this.ctx.clearRect(0, 0, this._width, this.height);
			this.ctx.restore();
			
			if (!this.options.isEnabled || !this._bounds) return;
			
			this.ctx.fillStyle = this.backgroundColor;
			this.ctx.fillRect(-(this._width - this._offset), 0, this._width * 2, this.height);

			var pixelLength = this.toSeconds(1); // yhden pikselin esittämä aika
			
			// Haetaan tämän ajan hetken sijanti
			var currentPos = this.currentPosition();

			// Piirretään alueet, joissa on tallennetta
			this.ctx.fillStyle = this.recAreaColor;
			for (var i = 0; i < this._bounds.length; i += 2) {
				var begin = this._bounds[i];
				var end = this._bounds[i + 1];
				this.ctx.fillRect(this.toPixels((begin - this._begin)), this.height - 10,
								  this.toPixels(end - begin), 6);
			}
			
			// Piirretään mahdollinen aikajanasta valittu alue
			if (this._rangeBegin !== -1 || this._rangeEnd !== -1) {
				var beginRangePos = this._rangeBegin !== -1 ? this.pixelPosition(this._rangeBegin) : 0;
				var endRangePos = this._rangeEnd !== -1 ? this.pixelPosition(this._rangeEnd) : this._width;

				if (beginRangePos < endRangePos) {
					this.ctx.fillStyle = "rgba(255, 255, 255, 0.2)";
					this.ctx.fillRect(beginRangePos, 0, endRangePos - beginRangePos, this.height);
				}
			}

			var minutes; // Kuinka monen minuutin välein aikatekstit on	
			if (pixelLength < 0.05) minutes = 5 / 60;
			else if (pixelLength < 0.12) minutes = 0.25;
			else if (pixelLength < 0.2) minutes = 0.5;
			else if (pixelLength < 0.5) minutes = 1;
			else if (pixelLength < 1) minutes = 5;
			else if (pixelLength < 3) minutes = 15;
			else if (pixelLength < 8) minutes = 30;
			else if (pixelLength < 15) minutes = 60;
			else minutes = 180;
			
			var centerT = this.time();
						
			// Lasketaan ensimmäinen näytettävä aika..
			var start = Math.round(centerT - this.toSeconds(this.visibleWidth));
			start = (Math.round(start / (minutes * 60)) * (minutes * 60));
			
			// ..ja viimeinen
			var end = Math.round(start + this.toSeconds(this.visibleWidth * 2));
			
			this.ctx.save();
			
			function drawTime(time) {
				var x = that.toPixels(time - that._begin);
				var yPos = 16;
				
				that.ctx.fillStyle = that._timeTextColor;
				that.ctx.font = "14px Arial";
				
				var date = new Date(time * 1000);				
				
				var text = ('0' + date.getHours()).slice(-2);
				if (pixelLength < 15) text += ":" + ('0' + date.getMinutes()).slice(-2);
				var textWidth = that.ctx.measureText(text).width;
				
				var secondsText = (pixelLength < 0.44) ? ":" + ('0' + date.getSeconds()).slice(-2) : "";
				var secondsTextWidth = that.ctx.measureText(secondsText).width; // Väärällä fonttikoolla laskettu

				that.ctx.fillText(text, x - (textWidth + secondsTextWidth) / 2, yPos);
				if (secondsText.length > 0) {
					that.ctx.fillText(secondsText, x - (textWidth + secondsTextWidth) / 2 + textWidth, yPos);
				}		
			}
			
			// Piirretään ajat..
			this.ctx.fillStyle = this._timeTextColor;
			var that = this;
			
			var timeCount = 0;
			for (var time = start; time < end; time += minutes * 60) {
				drawTime(time);
				timeCount++;
			}
			
			// ..ja väliajat.. (viritys)
			if (minutes === 30 || minutes === 60 || minutes === 1 || minutes === 0.5) {
				var distance = this.toPixels(minutes * 60) / 2;
				this.ctx.globalAlpha = distance < 180 ? Math.max(0, (distance - 80) / 100) : 1;
				if (this.ctx.globalAlpha !== 0) for (var time = start + minutes * 60 / 2; time < end; time += minutes * 60) drawTime(time);
			} else if (minutes % 3 === 0) {
				var distance = this.toPixels(minutes * 60) / 3;
				this.ctx.globalAlpha = distance < 180 ? Math.max(0, (distance - 80) / 100) : 1;
				if (this.ctx.globalAlpha !== 0) for (var i = 1; i < 3; ++i) for (var time = start + (i * minutes * 60) / 3; time < end; time += minutes * 60) drawTime(time);
			} else if (minutes === 0.25) {
				var distance = this.toPixels(minutes * 60) / 4;
				this.ctx.globalAlpha = distance < 100 ? Math.max(0, (distance - 50) / 100) : 1;
				for (var i = 1; i < 3; ++i) for (var time = start + 5 * i; time < end; time += minutes * 60) drawTime(time);
			} else {
				var distance = this.toPixels(minutes * 60) / 5;
				this.ctx.globalAlpha = distance < 100 ? Math.max(0, (distance - 50) / 100) : 1;
				if (this.ctx.globalAlpha !== 0) for (var i = 1; i < 5; ++i) for (var time = start + (i * minutes * 60) / 5; time < end; time += minutes * 60) drawTime(time);
			}
			this.ctx.globalAlpha = 1;
			
			// Piirretään ajan tämän hetkisestä sijainnista alkaen (jos se on aikajanalla) musta tausta
			if (currentPos > 0 && currentPos < this.width()) {
				this.ctx.fillStyle = "rgba(0, 0, 0, 1.0)";
				if (this._live && !this.isSeeking) { // Live-tilassa voidaan musta tausta piirtää aina keskiosasta alkaen
					// Piirretään tausta
					this.ctx.save();
					this.ctx.setTransform(1, 0, 0, 1, 0, 0);
					this.ctx.fillRect((this.width() / 2) / this.scale, 0, this.visibleWidth * 5, this.height);
					this.ctx.fillStyle = this.noTimeColor;
					this.ctx.fillRect(-this._width, 0, this._width + (this.width() / 2) / this.scale, this.height);
					this.ctx.restore();
				} else {
					this.ctx.fillRect(currentPos, 0, (this.width() - currentPos) * 10, this.height); // Piirretään reilusti yli, ettei jää tyhjäksi ulospäin zoomatessa
					this.ctx.fillStyle = this.noTimeColor;
					this.ctx.fillRect(-this._width, 0, this._width + currentPos, this.height);
				}
			} else {
				this.ctx.fillStyle = this.noTimeColor;
				this.ctx.fillRect(-this._width, 0, this._width * 10, this.height);
			}
			
			this.ctx.restore();
		},
		
		drawOverlay: function () {
			this.overlayCtx.clearRect(0, 0, this.visibleWidth, this.height);
		
			if (!this.options.isEnabled) {
				this.overlayCtx.fillStyle = this._timeTextColor;
				var fontSize = 16;
				this.overlayCtx.font = fontSize + "px Arial";
				this.overlayCtx.fillText(this.options.disabledText,
					this.visibleWidth / 2 - this.overlayCtx.measureText(this.options.disabledText).width / 2,
					this.height / 2 + fontSize / 3);
			} else {
				// Piirretään pystyviiva toistokohdan näyttämiseen
				this.overlayCtx.fillStyle = this.markColor;
				this.overlayCtx.fillRect(this.visibleWidth / 2 - this.markWidth / 2, 0, this.markWidth, this.height);
			}

			// Piirretään gradientit päihin
			var gradientWidth = this.visibleWidth / 4;
			
			var left = this.overlayCtx.createLinearGradient(0, 0, gradientWidth, 0);
			left.addColorStop(0, this.gradientStartColor);
			left.addColorStop(1, this.gradientEndColor);
			this.overlayCtx.fillStyle = left;
			this.overlayCtx.fillRect(0, 0, gradientWidth, this.height);
			
			var right = this.overlayCtx.createLinearGradient(this.visibleWidth - gradientWidth, 0, this.visibleWidth, 0);
			right.addColorStop(0, this.gradientEndColor);
			right.addColorStop(1, this.gradientStartColor);
			this.overlayCtx.fillStyle = right;
			this.overlayCtx.fillRect(this.visibleWidth - gradientWidth, 0, gradientWidth, this.height);
		},
		
		drawState: function () {				
			this.stateCtx.clearRect(0, 0, 100, this.height);
			if (!this.options.isEnabled || this._state.length === 0) return;
			
			this.stateCtx.fillStyle = this._timeTextColor;
			this.stateCtx.fillText(this._state, 5, 15);
		},
		
		drawTime: function () {
			this.timeCtx.clearRect(0, 0, 80, this.height);
			if (!this.options.isEnabled || !this._time) return;
			
			var time = new Date(this.time() * 1000);
			this.timeCtx.fillStyle = this._timeTextColor;
			this.timeCtx.fillText(time.getMonth() + 1 + "/" + time.getDate() + "/" + time.getFullYear(), 0, 15);
			this.timeCtx.fillText(('0' + time.getHours()).slice(-2) + ":" + ('0' + time.getMinutes()).slice(-2) + ":" + ('0' + time.getSeconds()).slice(-2), 0, 30);
		},
	
		animateError: function () {
			var that = this;
			function updateTimeColor() {
				if (that.options.isError) {
					var time = new Date().getTime()
					var bg = parseInt((Math.sin(time * 3.14159265358979 / 1000) + 1.0) * 128);
					that._timeTextColor = "rgba(200, " + bg + ", " + bg + ", 0.8)";
				} else {
					that._timeTextColor = that.timeTextColor;
				}
				that.draw();
				that.drawTime();
				that.drawState();
				
				if (that.options.isError) requestAnimFrame(updateTimeColor);
			};
			updateTimeColor();
		}
	});

})(jQuery);